#!/usr/bin/env python3
import argparse
import pandas as pd
from skbio.stats.ordination import pcoa
import json
import re

# --------------------------------------------------
'''
pcoa_sfir.py
Perform PCoA on a distance matrix and report the results in a sfir file
-in: distance matrix tsv file
-template: sfir template file
-out: pcoa sfir file
'''
# --------------------------------------------------

def createSFIRpcoa(distance_matrix, sfir_template, output):
    # Read distance matrix
    dm_df = pd.read_csv(distance_matrix, sep='\t', index_col=None)
    dm_rownames = dm_df.iloc[:, 0]
    dm = dm_df.iloc[:, 1:].to_numpy()

    # Perform PCoA and get coordinates
    pcoa_coords = (
        pcoa(dm, number_of_dimensions=2)
        .samples[['PC1', 'PC2']]
        .set_index(dm_rownames)
        .reset_index()
        .rename(columns={dm_rownames.name: 'id', 'PC1': 'x', 'PC2': 'y'})
    )

    # Convert to JSON and send to sfir template
    pcoa_coords_json = {"sfir_pcoa": pcoa_coords.to_dict(orient='records')}
    pcoa_coords_json_str = json.dumps(pcoa_coords_json, ensure_ascii=False)
    with open(sfir_template, 'r') as f:
        sfir = f.read()
        pattern = r"JSON\.parse\('\{\"sfir_pcoa\":.*\}'\)"
        json_str_match = re.search(pattern, sfir, re.DOTALL)
        if json_str_match:
            sfir = re.sub(pattern, f"JSON.parse('{pcoa_coords_json_str}')", sfir, flags=re.DOTALL)
            with open(output, 'w') as o:
                o.write(sfir)
        else:
            print("No match found, binding pattern for JSON input in sfir template is incorrect")
        
if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Perform PCoA on a distance matrix and report the results in sfir file")
    parser.add_argument("-d", "--distance_matrix", help="distance matrix file", required=True)
    parser.add_argument("-t", "--sfir_template", help="sfir template file", required=True)
    parser.add_argument("-o", "--output", help="output filename", default="pcoa.sfir.html")  
    args = parser.parse_args()
    createSFIRpcoa(args.distance_matrix, args.sfir_template, args.output)
#example usage: pcoa_sfir.py -d CDS2.tab -t pcoa_sfir_template.html